@@ -25,7 +25,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
-#include <libaudit.h>
+#include "libaudit.h"
 #include "ellist.h"
 #include "interpret.h"
 
@@ -102,7 +102,7 @@ static char *escape(const char *tmp)
 static int parse_up_record(rnode* r)
 {
 	char *ptr, *buf, *saved=NULL;
-	int offset = 0;
+	unsigned int offset = 0;
 
 	// Potentially cut the record in two
 	ptr = strchr(r->record, AUDIT_INTERP_SEPARATOR);
@@ -359,7 +359,7 @@ void aup_list_clear(event_list_t* l)
 	l->e.serial = 0L;
 	free((char *)l->e.host);
 	l->e.host = NULL;
-	free(l->cwd);
+	free((void *)l->cwd);
 }
 
 /*int aup_list_get_event(event_list_t* l, au_event_t *e)
