@@ -50,6 +50,12 @@
 #include <sys/personality.h>
 #include <sys/prctl.h>
 #include <sched.h>
+#ifdef USE_FANOTIFY
+#include <linux/fanotify.h>
+#else
+#define FAN_ALLOW 1
+#define FAN_DENY 2
+#endif
 #include "auparse-defs.h"
 #include "gen_tables.h"
 
@@ -809,7 +815,7 @@ static char *print_escaped(const char *val)
 	return strdup(val); // Something is wrong with string, just send as is
 }
 
-static const char *print_escaped_ext(int type, const idata *id)
+static const char *print_escaped_ext(const idata *id)
 {
 	if (id->cwd) {
 		char *str1 = NULL, *str2, *str3 = NULL, *out = NULL;
@@ -820,8 +826,7 @@ static const char *print_escaped_ext(int type, const idata *id)
 			str1 = print_escaped(id->cwd);
 			if (!str1)
 				goto err_out;
-			asprintf(&str3, "%s/%s", str1, str2);
-			if (!str3)
+			if (asprintf(&str3, "%s/%s", str1, str2) < 0)
 				goto err_out;
 		} else {
 			// Check in case /home/../etc/passwd
@@ -1066,7 +1071,8 @@ static const char *print_socket_proto(const char *val)
 
 static const char *print_sockaddr(const char *val)
 {
-        int slen, rc = 0;
+        size_t slen;
+        int rc = 0;
         const struct sockaddr *saddr;
         char name[NI_MAXHOST], serv[NI_MAXSERV];
         const char *host;
@@ -1345,7 +1351,7 @@ static const char *print_open_flags(const char *val)
 	size_t i;
 	unsigned int flags;
 	int cnt = 0;
-	char *out, buf[178];
+	char *out, buf[sizeof(open_flag_strings)+8];
 
 	errno = 0;
 	flags = strtoul(val, NULL, 16);
@@ -1383,7 +1389,7 @@ static const char *print_clone_flags(const char *val)
 {
 	unsigned int flags, i, clone_sig;
 	int cnt = 0;
-	char *out, buf[362]; // added 10 for signal name
+	char *out, buf[sizeof(clone_flag_strings)+16];// + 10 for signal name
 
 	errno = 0;
 	flags = strtoul(val, NULL, 16);
@@ -1490,8 +1496,8 @@ static const char *print_clock_id(const char *val)
 
 static const char *print_prot(const char *val, unsigned int is_mmap)
 {
-	unsigned int prot, i;
-	int cnt = 0, limit;
+	unsigned int prot, i, limit;
+	int cnt = 0;
 	char buf[144];
 	char *out;
 
@@ -1512,7 +1518,7 @@ static const char *print_prot(const char *val, unsigned int is_mmap)
 		limit = 4;
 	else
 		limit = 3;
-        for (i=0; i<limit; i++) {
+        for (i=0; i < limit; i++) {
                 if (prot_table[i].value & prot) {
                         if (!cnt) {
                                 strcat(buf,
@@ -1534,7 +1540,7 @@ static const char *print_mmap(const char *val)
 {
 	unsigned int maps, i;
 	int cnt = 0;
-	char buf[176];
+	char buf[sizeof(mmap_strings)+8];
 	char *out;
 
 	errno = 0;
@@ -1700,7 +1706,7 @@ static const char *print_recv(const char *val)
 {
 	unsigned int rec, i;
 	int cnt = 0;
-	char buf[234];
+	char buf[sizeof(recv_strings)+8];
 	char *out;
 
 	errno = 0;
@@ -1979,7 +1985,7 @@ static const char *print_shmflags(const char *val)
 {
 	unsigned int flags, partial, i;
 	int cnt = 0;
-	char *out, buf[32];
+	char *out, buf[sizeof(shm_mode_strings)+sizeof(ipccmd_strings)+8];
 
 	errno = 0;
 	flags = strtoul(val, NULL, 16);
@@ -2060,7 +2066,7 @@ static const char *print_umount(const char *val)
 {
 	unsigned int flags, i;
 	int cnt = 0;
-	char buf[64];
+	char buf[sizeof(umount_strings)+8];
 	char *out;
 
 	errno = 0;
@@ -2111,6 +2117,42 @@ static const char *print_ioctl_req(const char *val)
 	return out;
 }
 
+static const char *fanotify[3]= { "unknown", "allow", "deny" };
+static const char *aulookup_fanotify(unsigned s)
+{
+	switch (s)
+	{
+		default:
+			return fanotify[0];
+			break;
+		case FAN_ALLOW:
+			return fanotify[1];
+			break;
+		case FAN_DENY:
+			return fanotify[2];
+			break;
+	}
+}
+
+static const char *print_fanotify(const char *val)
+{
+        int res;
+
+	if (isdigit(*val)) {
+	        errno = 0;
+        	res = strtoul(val, NULL, 10);
+	        if (errno) {
+			char *out;
+			if (asprintf(&out, "conversion error(%s)", val) < 0)
+				out = NULL;
+	                return out;
+        	}
+
+	        return strdup(aulookup_fanotify(res));
+	} else
+		return strdup(val);
+}
+
 static const char *print_exit_syscall(const char *val)
 {
 	char *out;
@@ -2885,7 +2927,7 @@ unknown:
 			break;
 		case AUPARSE_TYPE_ESCAPED:
 		case AUPARSE_TYPE_ESCAPED_FILE:
-			out = print_escaped_ext(type, id);
+			out = print_escaped_ext(id);
 			break;
 		case AUPARSE_TYPE_ESCAPED_KEY:
 			out = print_escaped(id->val);
@@ -2980,6 +3022,9 @@ unknown:
 		case AUPARSE_TYPE_IOCTL_REQ:
 			out = print_ioctl_req(id->val);
 			break;
+		case AUPARSE_TYPE_FANOTIFY:
+			out = print_fanotify(id->val);
+			break;
 		case AUPARSE_TYPE_MAC_LABEL:
 		case AUPARSE_TYPE_UNCLASSIFIED:
 		default:
