@@ -89,6 +89,7 @@ _S(AUPARSE_TYPE_SESSION,	"ses"		)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"cap_pi"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"cap_pe"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"cap_pp"	)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"cap_pa"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"cap_fi"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"cap_fp"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"fp"		)
@@ -96,9 +97,14 @@ _S(AUPARSE_TYPE_CAP_BITMAP,	"fi"		)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"old_pp"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"old_pi"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"old_pe"	)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"old_pa"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"new_pp"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"new_pi"	)
 _S(AUPARSE_TYPE_CAP_BITMAP,	"new_pe"	)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"pp"		)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"pi"		)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"pe"		)
+_S(AUPARSE_TYPE_CAP_BITMAP,	"pa"		)
 _S(AUPARSE_TYPE_NFPROTO,	"family"	)
 _S(AUPARSE_TYPE_ICMPTYPE,	"icmptype"	)
 _S(AUPARSE_TYPE_PROTOCOL,	"proto"		)
@@ -133,4 +139,5 @@ _S(AUPARSE_TYPE_NETACTION,	"action"	)
 _S(AUPARSE_TYPE_MACPROTO,	"macproto"	)
 _S(AUPARSE_TYPE_ESCAPED,	"invalid_context")
 _S(AUPARSE_TYPE_IOCTL_REQ,	"ioctlcmd"	)
+_S(AUPARSE_TYPE_FANOTIFY,	"resp"		)
 
