@@ -19,6 +19,7 @@
  * Authors:
  *      Steve Grubb <sgrubb@redhat.com>
  *      Rickard E. (Rik) Faith <faith@redhat.com>
+ *      Richard Guy Briggs <rgb@redhat.com>
  */
 
 #include "config.h"
@@ -85,6 +86,7 @@ int _audit_permadded = 0;
 int _audit_archadded = 0;
 int _audit_syscalladded = 0;
 int _audit_exeadded = 0;
+int _audit_filterfsadded = 0;
 unsigned int _audit_elf = 0U;
 static struct libaudit_conf config;
 
@@ -517,6 +519,7 @@ int audit_set_backlog_wait_time(int fd, uint32_t bwt)
 int audit_reset_lost(int fd)
 {
 	int rc;
+	int seq;
 	struct audit_status s;
 
 	if ((audit_get_features() & AUDIT_FEATURE_BITMAP_LOST_RESET) == 0)
@@ -525,7 +528,7 @@ int audit_reset_lost(int fd)
 	memset(&s, 0, sizeof(s));
 	s.mask = AUDIT_STATUS_LOST;
 	s.lost = 0;
-	rc = audit_send(fd, AUDIT_SET, &s, sizeof(s));
+	rc = __audit_send(fd, AUDIT_SET, &s, sizeof(s), &seq);
 	if (rc < 0)
 		audit_msg(audit_priority(errno),
 			"Error sending lost reset request (%s)", 
@@ -672,7 +675,7 @@ int audit_request_signal_info(int fd)
 
 int audit_update_watch_perms(struct audit_rule_data *rule, int perms)
 {
-	int i, done=0;
+	unsigned int i, done=0;
 
 	if (rule->field_count < 1)
 		return -1;
@@ -835,7 +838,7 @@ int audit_make_equivalent(int fd, const char *mount_point,
 }
 
 /*
- * This function will retreive the loginuid or -1 if there
+ * This function will retrieve the loginuid or -1 if there
  * is an error.
  */
 uid_t audit_getloginuid(void)
@@ -899,7 +902,7 @@ int audit_setloginuid(uid_t uid)
 }
 
 /*
- * This function will retreive the login session or -2 if there
+ * This function will retrieve the login session or -2 if there
  * is an error.
  */
 uint32_t audit_get_session(void)
@@ -1466,6 +1469,14 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 		}
 	}
 
+	/* FS filter can be used only with FSTYPE field */
+	if (flags == AUDIT_FILTER_FS) {
+		uint32_t features = audit_get_features();
+		if ((features & AUDIT_FEATURE_BITMAP_FILTER_FS) == 0) {
+			return -EAU_FILTERNOSUPPORT;
+		}
+	}
+
 	rule->fields[rule->field_count] = field;
 	rule->fieldflags[rule->field_count] = op;
 	switch (field)
@@ -1580,7 +1591,8 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 			}
 			if (field == AUDIT_FILTERKEY &&
 				!(_audit_syscalladded || _audit_permadded ||
-				_audit_exeadded))
+				_audit_exeadded ||
+				_audit_filterfsadded))
                                 return -EAU_KEYDEP;
 			vlen = strlen(v);
 			if (field == AUDIT_FILTERKEY &&
@@ -1677,6 +1689,22 @@ int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
 				return -EAU_FILETYPEUNKNOWN;
 			}
 			break;
+		case AUDIT_FSTYPE:
+			if (!(flags == AUDIT_FILTER_FS))
+				return -EAU_FIELDUNAVAIL;
+			if (!(op == AUDIT_NOT_EQUAL || op == AUDIT_EQUAL))
+				return -EAU_OPEQNOTEQ;
+			if (isdigit((char)*(v))) 
+				rule->values[rule->field_count] = 
+					strtoul(v, NULL, 0);
+			else
+				rule->values[rule->field_count] = 
+					audit_name_to_fstype(v);
+			if ((int)rule->values[rule->field_count] == -1) {
+				return -EAU_FSTYPEUNKNOWN;
+			}
+			_audit_filterfsadded = 1;
+			break;
 		case AUDIT_ARG0...AUDIT_ARG3:
 			vlen = strlen(v);
 			if (isdigit((char)*(v))) 
