@@ -240,6 +240,8 @@ static const struct nv_list yes_no_values[] =
 
 const char *email_command = "/usr/lib/sendmail";
 static int allow_links = 0;
+static const char *config_dir = NULL;
+static char *config_file = NULL;
 
 
 void set_allow_links(int allow)
@@ -247,6 +249,26 @@ void set_allow_links(int allow)
 	allow_links = allow;
 }
 
+int set_config_dir(const char *val)
+{
+	config_dir = strdup(val);
+	if (config_dir == NULL)
+		return 1;
+	if (asprintf(&config_file, "%s/auditd.conf", config_dir) < 0)
+		return 1;
+	return 0;
+}
+
+const char *get_config_dir(void)
+{
+	/* This function is used to determine if audispd is started with
+	 * a -c parameter followed by the config_dir location. If we are
+	 * using the standard location, do not pass back a location. */
+	if (config_file && strcmp(config_file, CONFIG_FILE) == 0)
+		return NULL;
+	return config_dir;
+}
+
 /*
  * Set everything to its default value
 */
@@ -305,12 +327,14 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 
 	clear_config(config);
 	log_test = lt;
+	if (config_file == NULL)
+		config_file = strdup(CONFIG_FILE);
 
 	/* open the file */
 	mode = O_RDONLY;
 	if (allow_links == 0)
 		mode |= O_NOFOLLOW;
-	rc = open(CONFIG_FILE, mode);
+	rc = open(config_file, mode);
 	if (rc < 0) {
 		if (errno != ENOENT) {
 			audit_msg(LOG_ERR, "Error opening config file (%s)", 
@@ -318,7 +342,7 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 			return 1;
 		}
 		audit_msg(LOG_WARNING,
-			"Config file %s doesn't exist, skipping", CONFIG_FILE);
+			"Config file %s doesn't exist, skipping", config_file);
 		return 0;
 	}
 	fd = rc;
@@ -327,7 +351,7 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 	 * not symlink.
 	 */
 	audit_msg(LOG_DEBUG, "Config file %s opened for parsing", 
-			CONFIG_FILE);
+			config_file);
 	if (fstat(fd, &st) < 0) {
 		audit_msg(LOG_ERR, "Error fstat'ing config file (%s)", 
 			strerror(errno));
@@ -336,19 +360,19 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 	}
 	if (st.st_uid != 0) {
 		audit_msg(LOG_ERR, "Error - %s isn't owned by root", 
-			CONFIG_FILE);
+			config_file);
 		close(fd);
 		return 1;
 	}
 	if ((st.st_mode & S_IWOTH) == S_IWOTH) {
 		audit_msg(LOG_ERR, "Error - %s is world writable", 
-			CONFIG_FILE);
+			config_file);
 		close(fd);
 		return 1;
 	}
 	if (!S_ISREG(st.st_mode)) {
 		audit_msg(LOG_ERR, "Error - %s is not a regular file", 
-			CONFIG_FILE);
+			config_file);
 		close(fd);
 		return 1;
 	}
@@ -362,7 +386,7 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 		return 1;
 	}
 
-	while (get_line(f, buf, sizeof(buf), &lineno, CONFIG_FILE)) {
+	while (get_line(f, buf, sizeof(buf), &lineno, config_file)) {
 		// convert line into name-value pair
 		const struct kw_pair *kw;
 		struct nv_pair nv;
@@ -373,17 +397,17 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 			case 1: // not the right number of tokens.
 				audit_msg(LOG_ERR, 
 				"Wrong number of arguments for line %d in %s", 
-					lineno, CONFIG_FILE);
+					lineno, config_file);
 				break;
 			case 2: // no '=' sign
 				audit_msg(LOG_ERR, 
 					"Missing equal sign for line %d in %s", 
-					lineno, CONFIG_FILE);
+					lineno, config_file);
 				break;
 			default: // something else went wrong... 
 				audit_msg(LOG_ERR, 
 					"Unknown error for line %d in %s", 
-					lineno, CONFIG_FILE);
+					lineno, config_file);
 				break;
 		}
 		if (nv.name == NULL) {
@@ -394,7 +418,7 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 			fclose(f);
 			audit_msg(LOG_ERR,
 				"Not processing any more lines in %s",
-				CONFIG_FILE);
+				config_file);
 			return 1;
 		}
 
@@ -403,7 +427,7 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 		if (kw->name == NULL) {
 			audit_msg(LOG_ERR, 
 				"Unknown keyword \"%s\" in line %d of %s", 
-				nv.name, lineno, CONFIG_FILE);
+				nv.name, lineno, config_file);
 			fclose(f);
 			return 1;
 		}
@@ -413,7 +437,7 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 			audit_msg(LOG_ERR, 
 				"Keyword \"%s\" has invalid option "
 				"\"%s\" in line %d of %s", 
-				nv.name, nv.option, lineno, CONFIG_FILE);
+				nv.name, nv.option, lineno, config_file);
 			fclose(f);
 			return 1;
 		}
@@ -1073,7 +1097,9 @@ static int validate_email(const char *acct)
 
 	if ((ptr1 = strchr(acct, '@'))) {
 		char *ptr2;
-		struct hostent *t_addr;
+		int rc2;
+		struct addrinfo *ai;
+		struct addrinfo hints;
 
 		ptr2 = strrchr(acct, '.');        // get last dot - sb after @
 		if ((ptr2 == NULL) || (ptr1 > ptr2)) {
@@ -1082,21 +1108,26 @@ static int validate_email(const char *acct)
 			return 2;
 		}
 
-		t_addr = gethostbyname(ptr1+1);
-		if (t_addr == 0) {
+		memset(&hints, 0, sizeof(hints));
+		hints.ai_flags = AI_ADDRCONFIG | AI_CANONNAME;
+		hints.ai_socktype = SOCK_STREAM;
+
+		rc2 = getaddrinfo(ptr1+1, NULL, &hints, &ai);
+		freeaddrinfo(ai);
+		if (rc2 != 0) {
 			if ((h_errno == HOST_NOT_FOUND) ||
-					(h_errno == NO_RECOVERY)) {
-					audit_msg(LOG_ERR,
-				"validate_email: failed looking up host for %s",
-					ptr1+1);
-				// FIXME: gethostbyname is having trouble
-				// telling when we have a temporary vs permanent
-				// dns failure. So, for now, treat all as temp
-				return 1;
-			} else if (h_errno == TRY_AGAIN)
+						(h_errno == NO_RECOVERY)) {
+				audit_msg(LOG_ERR,
+			"validate_email: failed looking up host for %s (%s)",
+					ptr1+1, gai_strerror(rc2));
+				// FIXME: How can we tell that we truly have
+				// a permanent failure and what is that? For
+				// now treat all as temp failure.
+			} else if (h_errno == TRY_AGAIN) {
 				audit_msg(LOG_DEBUG,
 		"validate_email: temporary failure looking up domain for %s",
 					ptr1+1);
+			}
 			return 1;
 		}
 	}
@@ -1748,6 +1779,8 @@ void free_config(struct daemon_conf *config)
         free((void *)config->disk_error_exe);
         free((void *)config->krb5_principal);
         free((void *)config->krb5_key_file);
+        free((void *)config_dir);
+        free(config_file);
 }
 
 int resolve_node(struct daemon_conf *config)
@@ -1792,7 +1825,7 @@ int resolve_node(struct daemon_conf *config)
 				if (rc2 != 0) {
 					audit_msg(LOG_ERR,
 					"Cannot resolve hostname %s (%s)",
-					tmp_name, gai_strerror(rc));
+					tmp_name, gai_strerror(rc2));
 					rc = -1;
 					break;
 				}
@@ -1840,4 +1873,3 @@ int resolve_node(struct daemon_conf *config)
 				config->node_name);
 	return rc;
 }
-
