@@ -87,6 +87,11 @@ struct daemon_conf
 };
 
 void set_allow_links(int allow);
+
+/* Return 0 on success. */
+int set_config_dir(const char *val);
+const char *get_config_dir(void);
+
 int load_config(struct daemon_conf *config, log_test_t lt);
 void clear_config(struct daemon_conf *config);
 const char *audit_lookup_format(int fmt);
@@ -101,4 +106,3 @@ void shutdown_config(void);
 void free_config(struct daemon_conf *config);
 
 #endif
-
