@@ -542,8 +542,8 @@ void handle_event(struct auditd_event *e)
 		}
 	} else if (!config->write_logs)
 		send_ack(e, AUDIT_RMW_TYPE_ACK, "");
-	// FIXME: When logging is suspended, what should remote do?
-	// Should probably be new response type
+	else if (logging_suspended)
+		send_ack(e,AUDIT_RMW_TYPE_DISKERROR,"remote logging suspended");
 }
 
 static void send_ack(const struct auditd_event *e, int ack_type,
@@ -1062,7 +1062,7 @@ static void rotate_logs(unsigned int num_logs)
 	}
 }
 
-static int last_log = 1;
+static unsigned int last_log = 1;
 static void shift_logs(void)
 {
 	// The way this has to work is to start scanning from .1 up until
@@ -1345,7 +1345,7 @@ static void reconfigure(struct auditd_event *e)
 				// Likely errors: ENOMEM
 				do_disk_error_action("reconfig", saved_errno);
 			}
-			if(init_dispatcher(oconf)) {// dispatcher & qos is used
+			if(init_dispatcher(oconf,1)) {//dispatcher & qos is used
 				int saved_errno = errno;
 				audit_msg(LOG_WARNING,
 					"Could not start dispatcher %s"
@@ -1373,7 +1373,7 @@ static void reconfigure(struct auditd_event *e)
 				// Likely errors: ENOMEM
 				do_disk_error_action("reconfig", saved_errno);
 			}
-			if(init_dispatcher(oconf)) {// dispatcher & qos is used
+			if(init_dispatcher(oconf,1)) {// dispatcher& qos is used
 				int saved_errno = errno;
 				audit_msg(LOG_WARNING,
 					"Could not start dispatcher %s"
