@@ -68,7 +68,7 @@ static int fd = -1, pipefds[2] = {-1, -1};
 static struct daemon_conf config;
 static const char *pidfile = "/var/run/auditd.pid";
 static int init_pipe[2];
-static int do_fork = 1, opt_aggregate_only = 0;
+static int do_fork = 1, opt_aggregate_only = 0, config_dir_set = 0;
 static struct auditd_event *cur_event = NULL, *reconfig_ev = NULL;
 static int hup_info_requested = 0;
 static int usr1_info_requested = 0, usr2_info_requested = 0;
@@ -91,7 +91,9 @@ static const char *startup_states[] = {"disable", "enable", "nochange"};
  */
 static void usage(void)
 {
-	fprintf(stderr, "Usage: auditd [-f] [-l] [-n] [-s %s|%s|%s]\n",
+	fprintf(stderr,
+		"Usage: auditd [-f] [-l] [-n] [-s %s|%s|%s] "
+		"[-c <config_file>]\n",
 		startup_states[startup_disable],
 		startup_states[startup_enable],
 		startup_states[startup_nochange]);
@@ -557,6 +559,14 @@ int main(int argc, char *argv[])
 	struct sigaction sa;
 	struct rlimit limit;
 	int i, c, rc;
+	static const struct option opts[] = {
+		{"foreground", no_argument, NULL, 'f'},
+		{"allow_links", no_argument, NULL, 'l'},
+		{"disable_fork", no_argument, NULL, 'n'},
+		{"enable_state", required_argument, NULL, 's'},
+		{"config_file", required_argument, NULL, 'c'},
+		{NULL, 0, NULL, 0}
+	};
 	int opt_foreground = 0, opt_allow_links = 0;
 	enum startup_state opt_startup = startup_enable;
 	extern char *optarg;
@@ -571,7 +581,7 @@ int main(int argc, char *argv[])
 	struct ev_signal sigchld_watcher;
 
 	/* Get params && set mode */
-	while ((c = getopt(argc, argv, "aflns:")) != -1) {
+	while ((c = getopt_long(argc, argv, "flns:c:", opts, NULL)) != -1) {
 		switch (c) {
 		case 'f':
 			opt_foreground = 1;
@@ -596,6 +606,12 @@ int main(int argc, char *argv[])
 				usage();
 			}
 			break;
+ 		case 'c':
+			if (set_config_dir(optarg) != 0) {
+				usage();
+			}
+			config_dir_set = 1;
+			break;
 		default:
 			usage();
 		}
@@ -705,7 +721,7 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-	if (init_dispatcher(&config)) {
+	if (init_dispatcher(&config, config_dir_set)) {
 		if (pidfile)
 			unlink(pidfile);
 		tell_parent(FAILURE);
@@ -1054,4 +1070,3 @@ static char *getsubj(char *subj)
 	subj[num_read] = '\0';
 	return subj;
 }
-
