@@ -216,11 +216,11 @@ static void print_title_detailed(void)
 		case RPT_AVC:
 			printf("AVC Report\n");
 			printf(
-		  "========================================================\n");
+		  "===============================================================\n");
 			printf(
-		  "# date time comm subj syscall class permission obj event\n");
+		  "# date time comm subj syscall class permission obj result event\n");
 			printf(
-		  "========================================================\n");
+		  "===============================================================\n");
 			break;
 		case RPT_CONFIG:
 			printf("Config Change Report\n");
@@ -540,9 +540,9 @@ void print_per_event_item(llist *l)
 			break;
 		case RPT_LOGIN:
 			// who, addr, terminal, exe, success, event
-			// Special note...uid is used here because that is
-			// the way that the message works. This is because
-			// on failed logins, loginuid is not set.
+			// Special note...loginuid can be used here for
+			// successful logins. loginuid is not set on failed
+			// logins so acct is used in that situation.
 			safe_print_string(((l->s.success == S_FAILED) &&
 				l->s.acct) ? l->s.acct :
 				aulookup_uid(l->s.loginuid,
@@ -866,13 +866,13 @@ static void do_summary_output(void)
 			strftime(tmp, sizeof(tmp), "%x %T", btm);
 		else
 			strcpy(tmp, "?");
-		printf("%s.%03d - ", tmp, very_first_event.milli);
+		printf("%s.%03u - ", tmp, very_first_event.milli);
 		btm = localtime(&very_last_event.sec);
 		if (btm)
 			strftime(tmp, sizeof(tmp), "%x %T", btm);
 		else
 			strcpy(tmp, "?");
-		printf("%s.%03d\n", tmp, very_last_event.milli);
+		printf("%s.%03u\n", tmp, very_last_event.milli);
 	}
 	printf("Selected time for report: ");
 	{
@@ -899,7 +899,7 @@ static void do_summary_output(void)
 		if (end_time) 
 			printf("%s\n", tmp);
 		else 
-			printf("%s.%03d\n", tmp, very_last_event.milli);
+			printf("%s.%03u\n", tmp, very_last_event.milli);
 	}
 	printf("Number of changes in configuration: %lu\n", sd.changes);
 	printf("Number of changes to accounts, groups, or roles: %lu\n",
