@@ -114,6 +114,7 @@ void list_append(llist *l, lnode *node)
 
 	newnode->interp = node->interp;
 	newnode->mlen = node->mlen;
+	newnode->tlen = node->tlen;
 	newnode->type = node->type;
 	newnode->a0 = node->a0;
 	newnode->a1 = node->a1;
@@ -213,11 +214,11 @@ void list_clear(llist* l)
 	l->s.uuid = NULL;
 	free(l->s.vmname);
 	l->s.vmname = NULL;
-	free(l->s.tuid);
+	free((void *)l->s.tuid);
 	l->s.tuid = NULL;
-	free(l->s.teuid);
+	free((void *)l->s.teuid);
 	l->s.teuid = NULL;
-	free(l->s.tauid);
+	free((void *)l->s.tauid);
 	l->s.tauid = NULL;
 	l->s.exit = 0;
 	l->s.exit_is_set = 0;
