@@ -65,7 +65,7 @@ int event_exit_is_set = 0;
 int line_buffered = 0;
 int event_debug = 0;
 int checkpt_timeonly = 0;
-int extra_keys = 0, extra_labels = 0, extra_time = 0;
+int extra_keys = 0, extra_labels = 0, extra_obj2 = 0, extra_time = 0;
 const char *event_key = NULL;
 const char *event_filename = NULL;
 const char *event_exe = NULL;
@@ -92,7 +92,7 @@ S_TIME_END, S_TIME_START, S_TERMINAL, S_ALL_UID, S_EFF_UID, S_UID, S_LOGINID,
 S_VERSION, S_EXACT_MATCH, S_EXECUTABLE, S_CONTEXT, S_SUBJECT, S_OBJECT,
 S_PPID, S_KEY, S_RAW, S_NODE, S_IN_LOGS, S_JUST_ONE, S_SESSION, S_EXIT,
 S_LINEBUFFERED, S_UUID, S_VMNAME, S_DEBUG, S_CHECKPOINT, S_ARCH, S_FORMAT,
-S_EXTRA_TIME, S_EXTRA_LABELS, S_EXTRA_KEYS, S_ESCAPE };
+S_EXTRA_TIME, S_EXTRA_LABELS, S_EXTRA_KEYS, S_EXTRA_OBJ2, S_ESCAPE };
 
 static struct nv_pair optiontab[] = {
 	{ S_EVENT, "-a" },
@@ -107,6 +107,7 @@ static struct nv_pair optiontab[] = {
 	{ S_EXIT, "--exit" },
 	{ S_EXTRA_KEYS, "--extra-keys" },
 	{ S_EXTRA_LABELS, "--extra-labels" },
+	{ S_EXTRA_OBJ2, "--extra-obj2" },
 	{ S_EXTRA_TIME, "--extra-time" },
 	{ S_FILENAME, "-f" },
 	{ S_FILENAME, "--file" },
@@ -182,7 +183,7 @@ static struct nv_pair optiontab[] = {
 
 static int audit_lookup_option(const char *name)
 {
-        int i;
+        unsigned int i;
 
         for (i = 0; i < OPTION_NAMES; i++)
                 if (!strcmp(optiontab[i].name, name))
@@ -359,6 +360,15 @@ int check_params(int count, char *vars[])
         	                retval = -1;
 			}
 			break;
+		case S_EXTRA_OBJ2:
+			extra_obj2 = 1;
+			if (optarg) {
+				fprintf(stderr, 
+					"Argument is NOT required for %s\n",
+					vars[c]);
+        	                retval = -1;
+			}
+			break;
 		case S_EXTRA_TIME:
 			extra_time = 1;
 			if (optarg) {
