@@ -170,6 +170,7 @@ int extract_search_items(llist *l)
 			case AUDIT_MMAP:
 			case AUDIT_NETFILTER_CFG:
 			case AUDIT_PROCTITLE:
+			case AUDIT_KERN_MODULE:
 				// Nothing to parse
 				break;
 			case AUDIT_TTY:
@@ -269,7 +270,7 @@ static int parse_task_info(lnode *n, search_items *s)
 		if (errno)
 			return 21;
 		*term = ' ';
-		if (s->tauid) free(s->tauid);
+		if (s->tauid) free((void *)s->tauid);
 		s->tauid = lookup_uid("auid", s->loginuid);
 	}
 	// optionally get uid
@@ -287,7 +288,7 @@ static int parse_task_info(lnode *n, search_items *s)
 		if (errno)
 			return 24;
 		*term = ' ';
-		if (s->tuid) free(s->tuid);
+		if (s->tuid) free((void *)s->tuid);
 		s->tuid = lookup_uid("uid", s->uid);
 	}
 
@@ -947,42 +948,61 @@ static int parse_user(const lnode *n, search_items *s)
 			*term = saved;
 		}
 	}
-	if (event_subject) {
-		str = strstr(term, "vm-ctx=");
-		if (str != NULL) {
-			str += 7;
-			term = strchr(str, ' ');
-			if (term == NULL)
-				return 27;
-			*term = 0;
-			if (audit_avc_init(s) == 0) {
-				anode an;
-
-				anode_init(&an);
-				an.scontext = strdup(str);
-				alist_append(s->avc, &an);
-				*term = ' ';
-			} else
-				return 28;
+	if (n->type == AUDIT_VIRT_MACHINE_ID) {
+		if (event_subject) {
+			str = strstr(term, "vm-ctx=");
+			if (str != NULL) {
+				str += 7;
+				term = strchr(str, ' ');
+				if (term == NULL)
+					return 27;
+				*term = 0;
+				if (audit_avc_init(s) == 0) {
+					anode an;
+	
+					anode_init(&an);
+					an.scontext = strdup(str);
+					alist_append(s->avc, &an);
+					*term = ' ';
+				} else
+					return 28;
+			}
 		}
-	}
-	if (event_object) {
-		str = strstr(term, "img-ctx=");
-		if (str != NULL) {
-			str += 8;
-			term = strchr(str, ' ');
-			if (term == NULL)
-				return 29;
-			*term = 0;
-			if (audit_avc_init(s) == 0) {
-				anode an;
+		if (event_object) {
+			str = strstr(term, "img-ctx=");
+			if (str != NULL) {
+				str += 8;
+				term = strchr(str, ' ');
+				if (term == NULL)
+					return 29;
+				*term = 0;
+				if (audit_avc_init(s) == 0) {
+					anode an;
 
-				anode_init(&an);
-				an.tcontext = strdup(str);
-				alist_append(s->avc, &an);
-				*term = ' ';
-			} else
-				return 30;
+					anode_init(&an);
+					an.tcontext = strdup(str);
+					alist_append(s->avc, &an);
+					*term = ' ';
+				} else
+					return 30;
+			}
+		}
+	} else if (n->type == AUDIT_VIRT_RESOURCE) {
+		if (event_filename) {
+			unsigned int incr = 6;
+			str = strstr(term, " path=");
+			if (str == NULL) {
+				incr = 10;
+				str = strstr(term, " new-disk=");
+			}
+			if (str != NULL) {
+				int rc;
+				str += incr;
+				rc = common_path_parser(s, str);
+				if (rc)
+					return rc;
+				term = str;
+			}
 		}
 	}
 	// optionally get uid - some records the second uid is what we want.
@@ -1009,7 +1029,7 @@ static int parse_user(const lnode *n, search_items *s)
 			if (errno)
 				return 15;
 			*term = saved;
-			if (s->tuid) free(s->tuid);
+			if (s->tuid) free((void *)s->tuid);
 			s->tuid = lookup_uid("uid", s->uid);
 		}
 	}
