@@ -60,8 +60,7 @@ static int debug = 0;
 
 /* List of events */
 enum event_type {
-	ET_NONE = 0, ET_START, ET_STOP, ET_MACHINE_ID, ET_AVC, ET_RES, ET_ANOM,
-	ET_DOWN
+	ET_NONE = 0, ET_START, ET_STOP, ET_MACHINE_ID, ET_AVC, ET_RES, ET_ANOM
 };
 struct record_id {
 	time_t time;
@@ -118,7 +117,7 @@ struct event *event_alloc(void)
 void event_free(struct event *event)
 {
 	if (event) {
-		free(event->user);
+		free((void *)event->user);
 		free(event->uuid);
 		free(event->name);
 		free(event->reason);
@@ -291,10 +290,6 @@ auparse_state_t *init_auparse(void)
 	} else if (file) {
 		au = auparse_init(AUSOURCE_FILE, file);
 	} else {
-		if (getuid()) {
-			fprintf(stderr,
-			   "You probably need to be root for this to work\n");
-		}
 		au = auparse_init(AUSOURCE_LOGS, NULL);
 	}
 	if (au == NULL) {
@@ -303,73 +298,6 @@ auparse_state_t *init_auparse(void)
 	return au;
 }
 
-/* Create a criteria to search for the virtualization related records */
-int create_search_criteria(auparse_state_t *au)
-{
-	char *error = NULL;
-	char expr[1024];
-	snprintf(expr, sizeof(expr),
-		"(\\record_type >= %d && \\record_type <= %d)",
-		AUDIT_FIRST_VIRT_MSG, AUDIT_LAST_VIRT_MSG);
-	if (ausearch_add_expression(au, expr, &error, AUSEARCH_RULE_CLEAR)) {
-		fprintf(stderr, "Criteria error: %s\n", error);
-		free(error);
-		return 1;
-	}
-	if (uuid) {
-		if (ausearch_add_item(au, "uuid", "=", uuid,
-					AUSEARCH_RULE_AND)) {
-			fprintf(stderr, "Criteria error: uuid\n");
-			return 1;
-		}
-	}
-	if (vm) {
-		if (ausearch_add_interpreted_item(au, "vm", "=", vm,
-					AUSEARCH_RULE_AND)) {
-			fprintf(stderr, "Criteria error: id\n");
-			return 1;
-		}
-	}
-	if (all_events_flag || summary_flag) {
-		if (ausearch_add_item(au, "type", "=", "AVC",
-					AUSEARCH_RULE_OR)) {
-			fprintf(stderr, "Criteria error: AVC\n");
-			return 1;
-		}
-		if (ausearch_add_item(au, "type", "=", "SYSTEM_SHUTDOWN",
-					AUSEARCH_RULE_OR)) {
-			fprintf(stderr, "Criteria error: shutdown\n");
-			return 1;
-		}
-		snprintf(expr, sizeof(expr),
-			"(\\record_type >= %d && \\record_type <= %d) ||"
-			"(\\record_type >= %d && \\record_type <= %d)",
-			AUDIT_FIRST_ANOM_MSG, AUDIT_LAST_ANOM_MSG,
-			AUDIT_FIRST_KERN_ANOM_MSG, AUDIT_LAST_KERN_ANOM_MSG);
-		if (ausearch_add_expression(au, expr, &error,
-					AUSEARCH_RULE_OR)) {
-			fprintf(stderr, "Criteria error: %s\n", error);
-			free(error);
-			return 1;
-		}
-	}
-	if (start_time) {
-		if (ausearch_add_timestamp_item(au, ">=", start_time, 0,
-					AUSEARCH_RULE_AND)) {
-			fprintf(stderr, "Criteria error: start_time\n");
-			return 1;
-		}
-	}
-	if (end_time) {
-		if (ausearch_add_timestamp_item(au, "<=", end_time, 0,
-					AUSEARCH_RULE_AND)) {
-			fprintf(stderr, "Criteria error: end_time\n");
-			return 1;
-		}
-	}
-	return 0;
-}
-
 /* Extract the most common fields from virtualization-related records. */
 int extract_virt_fields(auparse_state_t *au, const char **p_uuid,
 		const char **p_user, time_t *p_time, const char **p_name,
@@ -417,7 +345,7 @@ error:
 				auparse_get_serial(au));
 	}
 	if (p_user)
-		free(*p_user);
+		free((void *) *p_user);
 	return 1;
 }
 
@@ -462,6 +390,40 @@ int add_proof(struct event *event, auparse_state_t *au)
 	return 0;
 }
 
+// This returns -1 if we don't want the event and 0 if we do
+int filter_event(auparse_state_t *au)
+{
+	extern time_t start_time, end_time;
+	time_t current = auparse_get_time(au);
+
+	if (start_time == 0 || current >= start_time) {
+		if (end_time == 0 || current <= end_time) {
+
+			if (vm) {
+				const char *v = auparse_find_field(au, "vm");
+				if (v) {
+					const char *v_text =
+						auparse_interpret_field(au);
+					if (v_text && strcmp(vm, v_text))
+						return -1;
+				}
+			}
+			if (uuid) {
+				const char *u = auparse_find_field(au, "uuid");
+				if (u) {
+					const char *u_text =
+						auparse_interpret_field(au);
+					if (u_text && strcmp(uuid, u_text))
+						return -1;
+				}
+			}
+			auparse_first_record(au);
+			return 0;
+		}
+	}
+	return -1;
+}
+
 /*
  * machine_id records are used to get the selinux context associated to a
  * guest.
@@ -469,10 +431,13 @@ int add_proof(struct event *event, auparse_state_t *au)
 int process_machine_id_event(auparse_state_t *au)
 {
 	time_t time;
-	const char *seclevel, *uuid, *name, *user = NULL;
+	const char *seclevel, *model, *uuid, *name, *user = NULL;
 	struct event *event;
 	int success;
 
+	if (filter_event(au))
+		return 0;
+
 	seclevel = get_seclevel(auparse_find_field(au, "vm-ctx"));
 	if (seclevel == NULL) {
 		if (debug)
@@ -480,12 +445,17 @@ int process_machine_id_event(auparse_state_t *au)
 					"MACHINE_ID event.\n");
 	}
 
+	// We only need to collect seclevel if model is selinux	
+	model = auparse_find_field(au, "model");
+	if (model && strcmp(model, "dac") == 0)
+		return 0;
+
 	if (extract_virt_fields(au, &uuid, &user, &time, &name, &success))
 		return 0;
 
 	event = event_alloc();
 	if (event == NULL) {
-		free(user);
+		free((void *)user);
 		return 1;
 	}
 	event->type = ET_MACHINE_ID;
@@ -541,7 +511,7 @@ int add_start_guest_event(auparse_state_t *au)
 
 	start = event_alloc();
 	if (start == NULL) {
-		free(user);
+		free((void *)user);
 		return 1;
 	}
 	start->type = ET_START;
@@ -605,14 +575,14 @@ int add_stop_guest_event(auparse_state_t *au)
 			fprintf(stderr, "Couldn't find the correlated start "
 					"record to the stop event.\n");
 		}
-		free(user);
+		free((void *)user);
 		return 0;
 	}
 
 	/* Create a new stop event */
 	stop = event_alloc();
 	if (stop == NULL) {
-		free(user);
+		free((void *)user);
 		return 1;
 	}
 	stop->type = ET_STOP;
@@ -626,7 +596,7 @@ int add_stop_guest_event(auparse_state_t *au)
 		stop->pid = auparse_get_field_int(au);
 	add_proof(stop, au);
 	if (list_append(events, stop) == NULL) {
-		free(user);
+		free((void *)user);
 		event_free(stop);
 		return 1;
 	}
@@ -643,6 +613,9 @@ int process_control_event(auparse_state_t *au)
 {
 	const char *op;
 
+	if (filter_event(au))
+		return 0;
+
 	op = auparse_find_field(au, "op");
 	if (op == NULL) {
 		if (debug)
@@ -677,7 +650,7 @@ int add_resource(auparse_state_t *au, const char *uuid, const char *user,
 		const char *res_type, const char *res)
 {
 	if (!is_resource(res)) {
-		free(user);
+		free((void *)user);
 		return 0;
 	}
 
@@ -719,8 +692,7 @@ int add_resource(auparse_state_t *au, const char *uuid, const char *user,
 }
 
 int update_resource(auparse_state_t *au, const char *uuid, time_t time,
-		const char *name, int success, const char *reason,
-		const char *res_type, const char *res)
+		int success, const char *res_type, const char *res)
 {
 	if (!is_resource(res) || !success)
 		return 0;
@@ -760,6 +732,9 @@ int process_resource_event(auparse_state_t *au)
 	const char *reason;
 	int success;
 
+	if (filter_event(au))
+		return 0;
+
 	/* Just skip this record if it failed to get some of the fields */
 	if (extract_virt_fields(au, &uuid, &user, &time, &name, &success))
 		return 0;
@@ -771,7 +746,7 @@ int process_resource_event(auparse_state_t *au)
 	if (res_type == NULL) {
 		if (debug)
 			fprintf(stderr, "Invalid resrc field.\n");
-		free(user);
+		free((void *)user);
 		return 0;
 	}
 
@@ -794,8 +769,8 @@ int process_resource_event(auparse_state_t *au)
 		if (res == NULL && debug) {
 			fprintf(stderr, "Failed to get %s field.\n", field);
 		} else {
-			rc += update_resource(au, uuid, time, name,
-					success, reason, res_type, res);
+			rc += update_resource(au, uuid, time, 
+					success, res_type, res);
 		}
 
 		/* Resource added */
@@ -805,7 +780,7 @@ int process_resource_event(auparse_state_t *au)
 			res = auparse_interpret_field(au);
 		if (res == NULL && debug) {
 			fprintf(stderr, "Failed to get %s field.\n", field);
-			free(user);
+			free((void *)user);
 		} else {
 			rc += add_resource(au, uuid, user, time, name, success,
 					reason, res_type, res);
@@ -820,9 +795,9 @@ int process_resource_event(auparse_state_t *au)
 	} else if (debug) {
 		fprintf(stderr, "Found an unknown resource: %s.\n",
 				res_type);
-		free(user);
+		free((void *)user);
 	} else
-		free(user);
+		free((void *)user);
 	return rc;
 }
 
@@ -835,23 +810,25 @@ struct event *get_machine_id_by_seclevel(const char *seclevel)
 	for (it = events->tail; it; it = it->prev) {
 		struct event *event = it->data;
 		if (event->type == ET_MACHINE_ID &&
-		    event->seclevel != NULL &&
-		    strcmp(event->seclevel, seclevel) == 0) {
-			machine_id = event;
-			break;
+		    event->seclevel != NULL) {
+			if (strcmp(event->seclevel, seclevel) == 0) {
+				machine_id = event;
+				break;
+			}
 		}
 	}
 
 	return machine_id;
 }
 
-int process_avc_selinux_context(auparse_state_t *au, const char *context)
+/* AVC records are correlated to guest through the selinux context. */
+int process_avc_selinux(auparse_state_t *au)
 {
 	const char *seclevel, *user = NULL;
 	struct event *machine_id, *avc;
 	time_t time;
 
-	seclevel = get_seclevel(auparse_find_field(au, context));
+	seclevel = get_seclevel(auparse_find_field(au, "scontext"));
 	if (seclevel == NULL) {
 		if (debug) {
 			fprintf(stderr, "Security context not found "
@@ -866,16 +843,16 @@ int process_avc_selinux_context(auparse_state_t *au, const char *context)
 	machine_id = get_machine_id_by_seclevel(seclevel);
 	if (machine_id == NULL) {
 		if (debug) {
-			fprintf(stderr, "Couldn't get the security "
-					"level from the AVC event.\n");
+			fprintf(stderr, "Couldn't get the machine id "
+				"based on security level in AVC event.\n");
 		}
-		free(user);
+		free((void *)user);
 		return 0;
 	}
 
 	avc = event_alloc();
 	if (avc == NULL) {
-		free(user);
+		free((void *)user);
 		return 1;
 	}
 	avc->type = ET_AVC;
@@ -916,19 +893,6 @@ int process_avc_selinux_context(auparse_state_t *au, const char *context)
 	return 0;
 }
 
-/* AVC records are correlated to guest through the selinux context. */
-int process_avc_selinux(auparse_state_t *au)
-{
-	const char **context;
-	const char *contexts[] = { "tcontext", "scontext", NULL };
-
-	for (context = contexts; context && *context; context++) {
-		if (process_avc_selinux_context(au, *context))
-			return 1;
-	}
-	return 0;
-}
-
 #ifdef WITH_APPARMOR
 int process_avc_apparmor_source(auparse_state_t *au)
 {
@@ -955,12 +919,7 @@ int process_avc_apparmor_source(auparse_state_t *au)
 	for (it = events->tail; it; it = it->prev) {
 		struct event *event = it->data;
 		if (event->success) {
-			if (event->type == ET_DOWN) {
-				/* It's just possible to find a matching guest
-				 * session in the current host session.
-				 */
-				break;
-			} else if (event->type == ET_RES &&
+			if (event->type == ET_RES &&
 			           event->end == 0 &&
 			           event->res != NULL &&
 		                   strcmp(target, event->res) == 0) {
@@ -1077,8 +1036,6 @@ int process_avc_apparmor_target(auparse_state_t *au)
 				} else if (event->type == ET_STOP) {
 					break;
 				}
-			} else if (event->type == ET_DOWN) {
-				break;
 			}
 		}
 	}
@@ -1143,9 +1100,11 @@ int process_avc_apparmor(auparse_state_t *au)
 
 int process_avc(auparse_state_t *au)
 {
+	if (filter_event(au))
+		return 0;
+
 	/* Check if it is a SELinux AVC record */
-	if (auparse_find_field(au, "tcontext")) {
-		auparse_first_record(au);
+	if (auparse_find_field(au, "scontext")) {
 		return process_avc_selinux(au);
 	}
 
@@ -1170,6 +1129,9 @@ int process_anom(auparse_state_t *au)
 	list_node_t *it;
 	struct event *anom, *start = NULL;
 
+	if (filter_event(au))
+		return 0;
+
 	/* An anomaly record is correlated to a guest by the process id */
 	if (auparse_find_field(au, "pid")) {
 		pid = auparse_get_field_int(au);
@@ -1251,7 +1213,7 @@ int process_anom(auparse_state_t *au)
 
 	anom = event_alloc();
 	if (anom == NULL) {
-		free(user);
+		free((void *)user);
 		return 1;
 	}
 	anom->type = ET_ANOM;
@@ -1269,48 +1231,6 @@ int process_anom(auparse_state_t *au)
 	return 0;
 }
 
-int process_shutdown(auparse_state_t *au)
-{
-	const char *user = NULL;
-	time_t time = 0;
-	struct event *down;
-	list_node_t *it;
-	int success = 0;
-
-	if (extract_virt_fields(au, NULL, &user, &time, NULL, &success))
-		return 0;
-
-	for (it = events->tail; it; it = it->prev) {
-		struct event *event = it->data;
-		if (event->success) {
-			if (event->type == ET_START || event->type == ET_RES) {
-				if (event->end == 0) {
-					event->end = time;
-					add_proof(event, au);
-				}
-			} else if (event->type == ET_DOWN) {
-				break;
-			}
-		}
-	}
-
-	down = event_alloc();
-	if (down == NULL) {
-		free(user);
-		return 1;
-	}
-	down->type = ET_DOWN;
-	down->user = user;
-	down->start = time;
-	down->success = success;
-	add_proof(down, au);
-	if (list_append(events, down) == NULL) {
-		event_free(down);
-		return 1;
-	}
-	return 0;
-}
-
 /* Convert record type to a string */
 const char *get_rec_type(struct event *e)
 {
@@ -1324,13 +1244,13 @@ const char *get_rec_type(struct event *e)
 	case ET_STOP:
 		return "stop";
 	case ET_RES:
-		return "res";
+		return "resrc";
 	case ET_AVC:
 		return "avc";
 	case ET_ANOM:
 		return "anom";
-	case ET_DOWN:
-		return "down";
+	default:
+		break;
 	}
 
 	snprintf(buf, sizeof(buf), "%d", e->type);
@@ -1381,13 +1301,13 @@ void print_event(struct event *event)
 		return;
 	/* The type of event is shown only when all records are shown */
 	if (all_events_flag)
-		printf("%-5.5s ", get_rec_type(event));
+		printf("%-6.5s ", get_rec_type(event));
 
 	/* Print common fields */
-	printf("%-25.25s", N(event->name));
+	printf("%-22.22s", N(event->name));
 	if (uuid_flag)
 		printf("\t%-36.36s", N(event->uuid));
-	printf("\t%-11.11s\t%-35.35s", N(event->user),
+	printf("\t%-11.11s\t%-32.32s", N(event->user),
 			get_time_period(event));
 
 	/* Print type specific fields */
@@ -1418,10 +1338,11 @@ void print_event(struct event *event)
 		printf("    Proof:");
 		for (i = 0; i < len; i++) {
 			if (event->proof[i].time) {
-				printf("%s %ld.%03u:%lu",
+				//printf("%s %ld.%03u:%lu",
+				printf("%s%lu",
 					(first) ? "" : ",",
-					event->proof[i].time,
-					event->proof[i].milli,
+//					event->proof[i].time,
+//					event->proof[i].milli,
 					event->proof[i].serial);
 				first = 0;
 			}
@@ -1446,8 +1367,7 @@ void print_summary(void)
 {
 	/* Summary numbers */
 	time_t stime = 0, etime = 0;
-	long start = 0, stop = 0, res = 0, avc = 0, anom = 0,
-	     shutdown = 0, failure = 0;
+	long start = 0, stop = 0, res = 0, avc = 0, anom = 0, failure = 0;
 	char start_buf[32], end_buf[32];
 
 	/* Calculate summary */
@@ -1476,8 +1396,7 @@ void print_summary(void)
 			case ET_ANOM:
 				anom++;
 				break;
-			case ET_DOWN:
-				shutdown++;
+			default:
 				break;
 			}
 		}
@@ -1519,7 +1438,6 @@ void print_summary(void)
 	printf("Number of resource assignments: %ld\n", res);
 	printf("Number of related AVCs:         %ld\n", avc);
 	printf("Number of related anomalies:    %ld\n", anom);
-	printf("Number of host shutdowns:       %ld\n", shutdown);
 	printf("Number of failed operations:    %ld\n", failure);
 }
 
@@ -1545,10 +1463,8 @@ int main(int argc, char **argv)
 	au = init_auparse();
 	if (au == NULL)
 		goto error;
-	if (create_search_criteria(au))
-		goto error;
 
-	while (ausearch_next_event(au) > 0) {
+	while (auparse_next_event(au) > 0) {
 		int err = 0;
 
 		switch(auparse_get_type(au)) {
@@ -1562,20 +1478,20 @@ int main(int argc, char **argv)
 			err = process_resource_event(au);
 			break;
 		case AUDIT_AVC:
-			err = process_avc(au);
+			if (all_events_flag || summary_flag)
+				err = process_avc(au);
 			break;
 		case AUDIT_FIRST_ANOM_MSG ... AUDIT_LAST_ANOM_MSG:
 		case AUDIT_FIRST_KERN_ANOM_MSG ... AUDIT_LAST_KERN_ANOM_MSG:
-			err = process_anom(au);
+			if (all_events_flag || summary_flag)
+				err = process_anom(au);
 			break;
-		case AUDIT_SYSTEM_SHUTDOWN:
-			err = process_shutdown(au);
+		default:
 			break;
 		}
 		if (err) {
 			goto unexpected_error;
 		}
-		auparse_next_event(au);
 	}
 
 	/* Show results */
